# *********************************************************************#
# Bladed Python Models API                                             #
# Copyright (c) DNV Services UK Limited (c) 2025. All rights reserved. #
# MIT License (see license file)                                       #
# *********************************************************************#


# coding: utf-8

from __future__ import annotations

from datetime import date, datetime  # noqa: F401
from enum import Enum, IntEnum

import os
import re  # noqa: F401
from typing import Annotated, Any, Dict, List, Literal, Optional, Set, Type, Union, Callable, Iterable  # noqa: F401
from pathlib import Path
from typing import TypeVar
Model = TypeVar('Model', bound='BaseModel')
StrBytes = Union[str, bytes]

from pydantic import AnyUrl, BaseModel, EmailStr, Field, validator, root_validator, Extra,PrivateAttr  # noqa: F401
from pydantic import ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.utils import ROOT_KEY
from json import encoder

from dnv_bladed_models.blade_stability_analysis_calculation import BladeStabilityAnalysisCalculation

from dnv_bladed_models.blade_stability_analysis_idling import BladeStabilityAnalysisIdling

from dnv_bladed_models.blade_stability_analysis_mode_of_operation import BladeStabilityAnalysisModeOfOperation

from dnv_bladed_models.blade_stability_analysis_mode_of_operation_insert import BladeStabilityAnalysisModeOfOperationInsert

from dnv_bladed_models.blade_stability_analysis_parked import BladeStabilityAnalysisParked



from .schema_helper import SchemaHelper 
from .models_impl import *

TBladeStabilityAnalysisModeOfOperationOptions = TypeVar('TBladeStabilityAnalysisModeOfOperationOptions', BladeStabilityAnalysisIdling, BladeStabilityAnalysisParked, BladeStabilityAnalysisModeOfOperationInsert, BladeStabilityAnalysisModeOfOperation, )

class BladeStabilityAnalysis(BladeStabilityAnalysisCalculation):
    r"""
    Defines a calculation which produces the damping and frequency of all the coupled rotor or blade modes plotted against wind speed for a single rotor.  This calculation allows analysis over a wider range of inflow conditions than the Cambell diagram calculation, rather than being constrained to normal operating conditions.
    
    Not supported yet.
    
    Attributes
    ----------
    SteadyCalculationType : Literal['BladeStabilityAnalysis'], default='BladeStabilityAnalysis', Not supported yet
        Defines the specific type of SteadyCalculation model in use.  For a `BladeStabilityAnalysis` object, this must always be set to a value of `BladeStabilityAnalysis`.
    
    ModeOfOperation : Union[BladeStabilityAnalysisIdling, BladeStabilityAnalysisParked, BladeStabilityAnalysisModeOfOperationInsert], Not supported yet
    
    Notes
    -----
    This model is not supported yet by the Bladed calculation engine.
    """
    _relative_schema_path: str = PrivateAttr('SteadyCalculation/BladeStabilityAnalysis.json')

    SteadyCalculationType: Literal['BladeStabilityAnalysis'] = Field(alias="SteadyCalculationType", default='BladeStabilityAnalysis', allow_mutation=False, const=True) # Not supported yet
    ModeOfOperation: Union[BladeStabilityAnalysisIdling, BladeStabilityAnalysisParked, BladeStabilityAnalysisModeOfOperationInsert] = Field(alias="ModeOfOperation", default=None, discriminator='BladeStabilityAnalysisModeOfOperationType') # Not supported yet

    class Config:
        extra = Extra.forbid
        validate_assignment = True
        allow_population_by_field_name = True
        pass


    @classmethod
    def parse_obj(cls: Type['Model'], obj: Any) -> 'Model':
        discriminated_props = [
            ('ModeOfOperation', 'BladeStabilityAnalysisModeOfOperationType'),
        ]
        discriminated_arrays = [
        ]
        prepare_model_dict(cls, obj, discriminated_props, discriminated_arrays)
        return super().parse_obj(obj)


    @classmethod
    def __get_validators__(cls):
        yield cls._factory


    @classmethod
    def _factory(cls, data):
        if isinstance(data, BladeStabilityAnalysis):
            return data
        if not isinstance(data, dict):
            raise TypeError('An object is required to define BladeStabilityAnalysis models')
        return BladeStabilityAnalysis.parse_obj(data)


    def _iter( # type: ignore
        self,
        **kwargs: Any
    ):
        if self.insert is not None:
            kwargs['exclude'] = None
            kwargs['include'] = set(['insert'])
        else:
            exclude: Optional[Set[str]] = kwargs.get('exclude', set())
            if exclude is None:
                exclude = self._find_unused_containers()
            else:
                exclude.update(self._find_unused_containers())
            kwargs['exclude'] = exclude
        return super()._iter(**kwargs)


    @property
    def ModeOfOperation_as_BladeStabilityAnalysisIdling(self) -> BladeStabilityAnalysisIdling:
        """
        Retrieves the value of ModeOfOperation guaranteeing it is a BladeStabilityAnalysisIdling; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        BladeStabilityAnalysisIdling
            A model object, guaranteed to be a BladeStabilityAnalysisIdling.

        Raises
        ------
        TypeError
            If the value is not a BladeStabilityAnalysisIdling.
        """
        return self.ModeOfOperation_as(BladeStabilityAnalysisIdling)


    @property
    def ModeOfOperation_as_BladeStabilityAnalysisParked(self) -> BladeStabilityAnalysisParked:
        """
        Retrieves the value of ModeOfOperation guaranteeing it is a BladeStabilityAnalysisParked; if it is not, an error is raised.

        Useful when using type-checking development tools, as it provides a concise way to retrieve a value and assign a variable with the correct type.

        Returns
        -------
        BladeStabilityAnalysisParked
            A model object, guaranteed to be a BladeStabilityAnalysisParked.

        Raises
        ------
        TypeError
            If the value is not a BladeStabilityAnalysisParked.
        """
        return self.ModeOfOperation_as(BladeStabilityAnalysisParked)


    @property
    def ModeOfOperation_as_inline(self) -> Union[BladeStabilityAnalysisIdling, BladeStabilityAnalysisParked]:
        """
        Retrieves the value of ModeOfOperation as a model object; if the value is specified with a '$insert', an error is raised.

        Returns
        -------
        Union[BladeStabilityAnalysisIdling, BladeStabilityAnalysisParked]
            A model object at the specified index, guaranteed to not be a '$insert'.

        Raises
        ------
        TypeError
            If the value is not one of the concrete types of BladeStabilityAnalysisModeOfOperation; i.e. it is specified with a '$insert'.
        """
        if isinstance(self.ModeOfOperation, BladeStabilityAnalysisModeOfOperationInsert) or self.ModeOfOperation.is_insert:
            raise TypeError(f"Expected ModeOfOperation value to be an in-line object, but it is currently in the '$insert' state.")
        return self.ModeOfOperation


    def ModeOfOperation_as(self, cls: Type[TBladeStabilityAnalysisModeOfOperationOptions])-> TBladeStabilityAnalysisModeOfOperationOptions:
        """
        Retrieves the value of ModeOfOperation, ensuring it is of the specified type.
        The specified type must be one of the valid concrete types of BladeStabilityAnalysisModeOfOperation, or the '$insert' type
        (in the case the model is defined to be inserted from an external resource).

        Useful when using type-checking development tools, as it provides a concise way to access a value with the correct type.

        Parameters
        ----------
        cls: Type[Union[BladeStabilityAnalysisIdling, BladeStabilityAnalysisParked, BladeStabilityAnalysisModeOfOperationInsert]]
            One of the valid concrete types of BladeStabilityAnalysisModeOfOperation, or the '$insert' type
            (in the case the model is defined to be inserted from an external resource).

        Returns
        -------
        TBladeStabilityAnalysisModeOfOperationOptions
            A model object of the specified type.

        Raises
        ------
        TypeError
            If the value is not of the specified type.

        Examples
        --------
        Get a reference to the value when it is one of the types of BladeStabilityAnalysisModeOfOperation:
        >>> val_obj = model_obj.ModeOfOperation_as(models.BladeStabilityAnalysisIdling)
        >>> val_obj = model_obj.ModeOfOperation_as(models.BladeStabilityAnalysisParked)

        Get a reference to the value, when it was specified with a '$insert' and read in from a file:
        >>> insert_obj = model_obj.ModeOfOperation_as(models.BladeStabilityAnalysisModeOfOperationInsert)
        """
        if not isinstance(self.ModeOfOperation, cls):
            raise TypeError(f"Expected ModeOfOperation of type '{cls.__name__}' but was type '{type(self.ModeOfOperation).__name__}'")
        return self.ModeOfOperation



    def _find_unused_containers(self) -> Set[str]:
        unused_containers: Set[str] = super()._find_unused_containers()
        return unused_containers


    def _is_unused(self) -> bool:
        """
        Returns true if the model has no user-supplied values. This indicates the object should not be rendered in the final JSON output.
        """
        candidate_fields = [x[1] for x in self.__dict__.items() if x[0] != 'SteadyCalculationType']
        return (len(candidate_fields) == 0 or all(map(lambda f: f is None, candidate_fields))) and super()._is_unused()


    def to_json(self, indent: Optional[int] = 2, **json_kwargs: Any) -> str:
        r"""
        Generates a JSON string representation of the model.

        Parameters
        ----------
        indent : int
            The whitespace indentation to use for formatting, as per json.dumps().

        Examples
        --------
        >>> model.to_json()
        Renders the full JSON representation of the model object.
        """

        json_kwargs['by_alias'] = True
        json_kwargs['exclude_unset'] = False
        json_kwargs['exclude_none'] = True
        if self.Schema is None:
            self.Schema = SchemaHelper.construct_schema_url(self._relative_schema_path)
        
        return super().json(indent=indent, **json_kwargs)


    @classmethod
    def from_file(cls: Type['Model'], path: Union[str, Path]) -> 'Model':
        r"""
        Loads a model from a given file path.

        Parameters
        ----------
        path : string
            The file path to the model.

        Returns
        -------
        BladeStabilityAnalysis
            The model object.

        Raises
        ------
        ValueError, ValidationError
            If the JSON document does not correctly describe the model according to the model schema.

        Examples
        --------
        >>> model = BladeStabilityAnalysis.from_file('/path/to/file')
        """
        
        return super().parse_file(path=path)


    @classmethod
    def from_json(cls: Type['Model'], b: StrBytes) -> 'Model':
        r"""
        Creates a model object from a JSON string.

        Parameters
        ----------
        b: StrBytes
            The JSON string describing the model.

        Returns
        -------
        BladeStabilityAnalysis
            The model object.

        Raises
        ------
        ValueError, ValidationError
            If the JSON document does not correctly describe the model according to the model schema.

        Examples
        --------
        >>> model = BladeStabilityAnalysis.from_json('{ ... }')
        """

        return super().parse_raw(
            b=b,
            content_type='application/json')
        

    @classmethod
    def from_dict(cls: Type['Model'], obj: Any) -> 'Model':
        r"""
        Creates a model object from a dict.
        
        Parameters
        ----------
        obj : Any
            The dictionary object describing the model.

        Returns
        -------
        BladeStabilityAnalysis
            The model object.

        Raises
        ------
        ValueError, ValidationError
            If the JSON document does not correctly describe the model according to the model schema.
        """
        
        return cls.parse_obj(obj=obj)


    def to_file(self, path: Union[str, Path]) -> None:
        r"""
        Writes the model as a JSON document to a file with UTF8 encoding.

        Parameters
        ----------                
        path : string
            The file path to which the model will be written.

        Examples
        --------
        >>> model.to_file('/path/to/file')
        """

        with open(file=path, mode='w', encoding="utf8") as output_file:
            output_file.write(self.to_json())


BladeStabilityAnalysis.update_forward_refs()
